<?php
  /**
   * This is the Master class which includes and instantiates all Core classes
   *
   * @author Macdonald Terrence Robinson
   * @package Core
   *
   */

  require_once dirname(__FILE__).'/statics/Session.php';
  require_once dirname(__FILE__).'/statics/Get.php';
  require_once dirname(__FILE__).'/statics/Post.php';
  require_once dirname(__FILE__).'/statics/Cookie.php';

  require_once dirname(__FILE__).'/statics/FileSystem.php';
  require_once dirname(__FILE__).'/statics/Utility.php';
  require_once dirname(__FILE__).'/statics/Parser.php';
  require_once dirname(__FILE__).'/statics/XML.php';

  require_once dirname(__FILE__).'/statics/URI.php';
  require_once dirname(__FILE__).'/statics/Router.php';

  require_once dirname(__FILE__).'/statics/SiteMap.php';
  require_once dirname(__FILE__).'/../Config.php';

  require_once dirname(__FILE__).'/statics/ErrorHandler.php';
  require_once dirname(__FILE__).'/statics/HTML.php';
  require_once dirname(__FILE__).'/statics/Cache.php';

  require_once dirname(__FILE__).'/statics/XSS.php';

  final class Master
  {
    private static $db = NULL;    
    private static $mainController = NULL;
    private static $currentController = NULL;

    public static function loadCoreClass($className, $arguments = array())
    {
      $path = Config::$siteDir.Config::$coreDir.$className.'.php';

      if(!file_exists($path))
        return $matches[0];
      
      if(!class_exists($className))
          require $path;

      return Master::initClass($className, $arguments);
    }

    public static function customUrlHandler()
    {
      if( str_replace(array('index.php', 'index.php/'),'',URI::getURL(true)) === Config::$siteURL)
        return;

      $testUrl = str_replace( Config::$siteURL.'index.php/', Config::$siteURL, URI::getURL(true));
      $testDir = str_replace( Config::$siteURL, Config::$siteDir, $testUrl);

      if(file_exists($testDir) && is_file($testDir))
      {
        header('Location:'.$testUrl.URI::getQueryString());
        exit;
      }
    }

    public static function init()
    {      
      Session::init();
      Post::init();
      Get::init();
      Cookie::init();

      Config::init();      
      URI::init();
      Router::init();
            
      XSS::init();
      
      self::customUrlHandler();

      if(Config::$enableAutoXSSFilter)
        if(count(Config::$globalKeysToFilter) > 0 )
          XSS::cleanGlobalKeys( Config::$globalKeysToFilter );

      if(Session::isExists('AdminLoggedIn') || Get::isExists('ajaxRequest') || Post::isExists('ajaxRequest'))
        Config::$enableCaching = false;

      if( Get::isExists('lang') )
      {
        Config::$siteLanguage = strtolower(Get::getByKey('lang'));
        Session::set('lang', Config::$siteLanguage);
      }
      else if(Session::isExists('lang'))
        Config::$siteLanguage = Session::getByKey('lang');

      Config::loadCustomConfig();

      Parser::init();
      SiteMap::init();

      $currentNode = SiteMap::getCurrentNode();

      if( $currentNode != NULL)
      {
        Router::$controller = (string)$currentNode['controller'];
        Router::$function = (string)$currentNode['function'];
      }
      
      Parser::initCurrentNodeAttributes();
      
      if(Config::$enableCaching)
      {        
        foreach (Config::$disableCaching as $key=>$value)
          Config::$disableCaching[$key] = strtolower(Config::$disableCaching[$key]);

        $cacheData = Cache::loadCache();

        if($cacheData !='')
        {
          echo $cacheData;
          return false;
        }
      }

      if(Config::$isSiteOffline)
      {
        self::loadController( Router::$controller, Config::$offlineFunctionName , Router::$params );
        return false;
      }

      if(Config::$dbConnect)
        self::connectToDB();      
      return true;
    }

    public static function connectToDB()
    {
      if(self::$db == NULL)
      {
        $dbSettings = array();

        $dbSettings['dbType'] = isset(Config::$dbSettings['dbType']) ? Config::$dbSettings['dbType'] : '';
        $dbSettings['odbcDriver'] = isset(Config::$dbSettings['odbcDriver']) ? Config::$dbSettings['odbcDriver'] : '';
        $dbSettings['dbHost'] = isset(Config::$dbSettings['dbHost']) ? Config::$dbSettings['dbHost'] : '';
        $dbSettings['dbName'] = isset(Config::$dbSettings['dbName']) ? Config::$dbSettings['dbName'] : '';
        $dbSettings['dbUser'] = isset(Config::$dbSettings['dbUser']) ? Config::$dbSettings['dbUser'] : '';
        $dbSettings['dbPassword'] = isset(Config::$dbSettings['dbPassword']) ? Config::$dbSettings['dbPassword'] : '';
        $dbSettings['connectionString'] = isset(Config::$dbSettings['connectionString']) ? Config::$dbSettings['connectionString'] : '';
        
        self::$db = self::loadCoreClass('DB', $dbSettings);
      }

      return self::$db;
    }

    /** @return MainController */
    public static function getMainController()
    {
      if(self::$mainController == NULL)
        self::$mainController = self::loadCoreClass('MainController');

      return self::$mainController;
    }

    /** @return BaseController */
    public static function getCurrentController()
    {      
      if(self::$currentController == NULL)
        self::$currentController = self::loadController(Router::$controller, Router::$function, Router::$params);

      return self::$currentController;
    }

    public static function start()
    {
      self::getCurrentController();      
    }

    /**
     * Loads a controller
     * @return MainController
     */
    public static function loadController($className, $functionName, $params = array())
    {      
      if($className == '')
        $className = Config::$defaultController;

      if($functionName == '')
        $functionName = Config::$defaultFunction;

      self::$currentController = Master::loadAndInitClass($className, $params,'', Config::$controllersDir);

      return Master::initFunction(self::$currentController, $functionName, $params);
    }

    public static function getClassFilePath($className)
    {
      $possibleDirs = array();
      $possibleDirs[] = Config::$coreDir;
      $possibleDirs[] = Config::$controllersDir;

      foreach ($possibleDirs as $key=>$value)
      {
        $filePath = self::getRealPath($value.$className.'.php');

        if( $filePath == '' )
          continue;

        return $filePath;
      }

      return '';
    }

    public static function loadAndInitClass($className, array $arguments = array(), $fromFile = '', $inDir = '')
    {      
      if(($inDir === '') && ($fromFile === ''))
        $filePath = self::getClassFilePath($className);
      else
      {
        if($fromFile == '')
          $filePath = $inDir.$className.'.php';
        else
          $filePath = $fromFile;

        if( ($filePath = self::getRealPath($filePath)) == '' )
        {
          //Master::getMainController()->redirectToHome();
          ErrorHandler::runTimeError("Error initializing class ( $className ) from file ( $filePath )");
          return false;
        }
      }
      
      require_once $filePath;

      return self::initClass($className, $arguments);
    }

    public static function initClass($className, array $arguments = array())
    {
      if(class_exists($className) )
      {
        try
        {
          if( count($arguments) < 2 )
          {
            $argument = (isset($arguments[0]))? $arguments[0] : '';
            return new $className($argument);
          }
          else
          {
            $class = new ReflectionClass($className);
            return $class->newInstanceArgs($arguments);
          }
        }
        catch(Exception $e)
        {
          $errorMessage = $e->getMessage();
          ErrorHandler::runTimeError($errorMessage);
        }
      }

      return NULL;
    }

    public static function initFunction( $object, $functionName, $arguments )
    {
      try
      {
        if( is_string($object) )
        {
          $reflection = new ReflectionClass($object);
          $method = $reflection->getMethod($functionName);
          return $method->invoke($object, $arguments);
        }
        else if( Utility::isFunctionInObject($object, $functionName) && ( count($arguments) < 2 ))
        {
          $argument = (isset($arguments[0]))? $arguments[0] : '';
          return $object->$functionName($argument);
        }
        else
        {
          $reflectionMethod = new ReflectionMethod( $object, $functionName );
          return $reflectionMethod->invokeArgs($object, $arguments);
        }
      }
      catch(Exception $e)
      {
        return self::getCurrentController()->catchAll();
        //$reflectionMethod = new ReflectionMethod( self::getCurrentController(), 'catchAll' );
        //return $reflectionMethod->invokeArgs(self::getCurrentController(), array()) ;
      }
    }
    
    public static function getRealPath( $path )
    {
      $segments = explode( Config::$directorySeperator, $path );

      $fileName = $segments[ count($segments)-1 ];
      $segments[ count($segments)-1 ] = '';
      $dir = implode(Config::$directorySeperator, $segments);

      $path = $dir.$fileName;

      $patterns = array();
      $patterns[] = $path;

      if( FileSystem::fileExists($path) )
        return $path;

      $path = $dir.strtolower($fileName);
      if( !in_array($path, $patterns) && ( FileSystem::fileExists( $path ) ))
        return $path;
      $patterns[] = $path;

      $path = $dir.ucfirst( strtolower($fileName) );
      if( !in_array($path, $patterns) && FileSystem::fileExists( $path ) )
        return $path;
      $patterns[] = $path;

      $path = $dir.strtoupper($fileName);
      if( !in_array($path, $patterns) && FileSystem::fileExists( $path ) )
        return $path;
      $patterns[] = $path;

      $path = $dir.ucfirst(strtolower($fileName));
      if( !in_array($path, $patterns) && FileSystem::fileExists( $path ) )
        return $path;
      $patterns[] = $path;

      return '';
    }

  }
?>
